"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var helper_plugin_utils_1 = require("@babel/helper-plugin-utils");
var template_1 = __importDefault(require("@babel/template"));
var t = __importStar(require("@babel/types"));
var plugin_syntax_jsx_1 = __importDefault(require("@babel/plugin-syntax-jsx"));
var ast_builders_1 = require("./utils/ast-builders");
var cache_1 = require("./utils/cache");
var css_prop_1 = require("./css-prop");
var styled_1 = require("./styled");
var cache = new cache_1.Cache();
exports.default = helper_plugin_utils_1.declare(function (api) {
    api.assertVersion(7);
    return {
        inherits: plugin_syntax_jsx_1.default,
        pre: function () {
            this.sheets = {};
            cache.initialize(this.opts);
            this.cache = cache;
        },
        visitor: {
            Program: {
                exit: function (path, state) {
                    if (state.compiledImports && !path.scope.getBinding('React')) {
                        // React is missing - add it in at the last moment!
                        path.unshiftContainer('body', template_1.default.ast("import * as React from 'react'"));
                    }
                },
            },
            ImportDeclaration: function (path, state) {
                if (path.node.source.value !== '@compiled/react') {
                    return;
                }
                // The presence of the module enables CSS prop
                state.compiledImports = {};
                // Go through each import and enable each found API
                path.get('specifiers').forEach(function (specifier) {
                    if (!state.compiledImports || !specifier.isImportSpecifier()) {
                        // Bail out early
                        return;
                    }
                    if (specifier.node.imported.name === 'styled') {
                        // Enable styled API with the local name
                        state.compiledImports.styled = specifier.node.local.name;
                        // Remove specifier
                        specifier.remove();
                    }
                });
                // Add the runtime entrypoint module
                path.insertBefore(t.importDeclaration([ast_builders_1.importSpecifier('ax'), ast_builders_1.importSpecifier('CC'), ast_builders_1.importSpecifier('CS')], t.stringLiteral('@compiled/react/runtime')));
                if (path.node.specifiers.length === 0) {
                    // No more imports - remove the whole lot!
                    path.remove();
                }
            },
            TaggedTemplateExpression: function (path, state) {
                var _a;
                if (!((_a = state.compiledImports) === null || _a === void 0 ? void 0 : _a.styled)) {
                    return;
                }
                styled_1.visitStyledPath(path, { state: state, parentPath: path });
            },
            CallExpression: function (path, state) {
                if (!state.compiledImports) {
                    return;
                }
                styled_1.visitStyledPath(path, { state: state, parentPath: path });
            },
            JSXOpeningElement: function (path, state) {
                if (!state.compiledImports) {
                    return;
                }
                css_prop_1.visitCssPropPath(path, { state: state, parentPath: path });
            },
        },
    };
});
//# sourceMappingURL=index.js.map